@@ -11,6 +11,11 @@
 #' between \eqn{x_{i-10}} and \eqn{x_{i+10}}, resulting in \code{10*2 + 1 = 21} observations falling within
 #' the window.
 #'
+#' @note
+#'
+#' If you wish to calculate the lower bound between several time series, it would be better to use the version
+#' registered with the 'proxy' package, since it includes some small optimizations. See the examples.
+#'
 #' @references
 #'
 #' Keogh E and Ratanamahatana CA (2005). ``Exact indexing of dynamic time warping.'' \emph{Knowledge and information systems}, \strong{7}(3),
@@ -30,22 +35,36 @@
 #'
 #' d.lbk <= d.dtw
 #'
+#' # Calculating the LB between several time series using the 'proxy' package
+#' # (notice how both argments must be lists)
+#' D.lbk <- proxy::dist(CharTraj[1], CharTraj[2:5], method = "LB_Keogh",
+#'                      window.size = 20, norm = "L2")
+#'
+#' # Corresponding true DTW distance
+#' # (see dtwclust-package description for an explanation of DTW2)
+#' D.dtw <- proxy::dist(CharTraj[1], CharTraj[2:5], method = "DTW2",
+#'                      window.type = "slantedband", window.size = 20)
+#'
+#' D.lbk <= D.dtw
+#'
 #' @param x A time series.
 #' @param y A time series with the same length as \code{x}.
 #' @param window.size Window size for envelope calculation. See details.
 #' @param norm Pointwise distance. Either \code{L1} for Manhattan distance or \code{L2} for Euclidean.
+#' @param lower.env Optionally, a pre-computed lower envelope for \strong{\code{y}} can be provided.
+#' @param upper.env Optionally, a pre-computed upper envelope for \strong{\code{y}} can be provided.
 #'
 #' @return A list with: \itemize{
 #'   \item \code{d}: The lower bound of the DTW distance.
-#'   \item \code{upper.env}: The time series of the upper envelope.
-#'   \item \code{lower.env}: The time series of the lower envelope.
+#'   \item \code{upper.env}: The time series of \code{y}'s upper envelope.
+#'   \item \code{lower.env}: The time series of \code{y}'s lower envelope.
 #' }
 #'
 #' @export
 #' @importFrom caTools runmax
 #' @importFrom caTools runmin
 
-lb_keogh <- function(x, y, window.size = NULL, norm = "L1") {
+lb_keogh <- function(x, y, window.size = NULL, norm = "L1", lower.env = NULL, upper.env = NULL) {
 
      norm <- match.arg(norm, c("L1", "L2"))
 
@@ -62,8 +81,19 @@ lb_keogh <- function(x, y, window.size = NULL, norm = "L1") {
           stop("The width of the window should not exceed the length of the series")
 
      ## NOTE: the 'window.size' definition varies betwen 'dtw' and 'runmax/min'
-     upper.env <- caTools::runmax(y, window.size*2+1, endrule="constant")
-     lower.env <- caTools::runmin(y, window.size*2+1, endrule="constant")
+     if (is.null(lower.env)) {
+          lower.env <- caTools::runmin(y, window.size*2+1, endrule="constant")
+     } else {
+          if (length(lower.env) != length(y))
+               stop("Length mismatch between 'y' and its lower envelope")
+     }
+
+     if (is.null(upper.env)) {
+          upper.env <- caTools::runmax(y, window.size*2+1, endrule="constant")
+     } else {
+          if (length(upper.env) != length(y))
+               stop("Length mismatch between 'y' and its upper envelope")
+     }
 
      D <- rep(0, length(x))
 
@@ -162,15 +192,19 @@ lb_keogh_loop <- function(x, y=NULL, ...) {
                   })
 
      if (force.symmetry) {
-          ind.tri <- lower.tri(DD)
+          if (nrow(DD) != ncol(DD)) {
+               warning("Unable to force symmetry. Resulting distance matrix is not square.")
+          } else {
+               ind.tri <- lower.tri(DD)
 
-          new.low.tri.vals <- t(DD)[ind.tri]
-          indCorrect <- DD[ind.tri] > new.low.tri.vals
-          new.low.tri.vals[indCorrect] <- DD[ind.tri][indCorrect]
+               new.low.tri.vals <- t(DD)[ind.tri]
+               indCorrect <- DD[ind.tri] > new.low.tri.vals
+               new.low.tri.vals[indCorrect] <- DD[ind.tri][indCorrect]
 
-          DD[ind.tri] <- new.low.tri.vals
-          DD <- t(DD)
-          DD[ind.tri] <- new.low.tri.vals
+               DD[ind.tri] <- new.low.tri.vals
+               DD <- t(DD)
+               DD[ind.tri] <- new.low.tri.vals
+          }
      }
 
      attr(DD, "class") <- "crossdist"
